<?php

namespace App\Console\Commands;

use Aws\S3\Exception\S3Exception;
use Illuminate\Console\Command;
use Spatie\MediaLibrary\Exceptions\FileCannotBeAdded;
use Spatie\MediaLibrary\Models\Media;

class MigrateMedia extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'migrate:media';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Migrate all media to S3.';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        if ($this->confirm('Do you wish to continue?')) {
            ini_set('max_execution_time', '0');
            $allMedia = Media::query()
                ->where('disk', '=', 'public')
                ->get();
            dd($allMedia);

            $bar = $this->output->createProgressBar(count($allMedia));
            $bar->start();
            foreach ($allMedia as $media) {
                if (isset($media->model)) {
                    $relatedModel = $media->model;
                    $sourceImg = $media->getFullUrl();
                    try {
                        $relatedModel->addMediaFromUrl($sourceImg)
                            ->toMediaCollection($media->collection_name);
                        $this->info('Updated media id ' . ${$media->id} . ' for model ' . $media->model_type .
                            ', with id ('. $media->model_id . ') and url ' . $sourceImg);
                    } catch (FileCannotBeAdded $e) {
                        \Log::debug('File copy error. Error: ' . $e->getMessage());
                    } catch (S3Exception $e) {
                        \Log::debug('File copy error. Error: ' . $e->getMessage());
                    } catch (Exception $e) {
                        $this->error('Something went wrong!');
                        \Log::debug('File copy error. Error: ' . $e->getMessage());
                    }
                } else {
                    $this->warn('No media to migrate.');
                }
                $bar->advance();
            }
            $bar->finish();
        }
        $this->info('Migration finished');
    }
}
