<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Schedule;
use App\ServiceType;
use App\TaskGroup;
use Illuminate\Http\Request;

class AirlineServiceComplianceController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @param int $airline_id
     * @param int $service_type_id
     * @param array|null $period
     * @return array
     */
    public function index(int $airline_id, int $service_type_id, $fleet_ids): array
    {
        $total = Schedule::query()
            ->totalScheduled($airline_id, $service_type_id, $fleet_ids)
            ->get();

        $overdue = Schedule::query()
            //->where('schedules.date', '<', now()->subDays(16)->format('Y-m-d')) //
            ->when($airline_id === 58 && $service_type_id == 3, function($q) {
                return $q->where('schedules.date', '<', now()->subDays(16)->format('Y-m-d'));
            })
            ->where('schedules.date', '<=', now()->format('Y-m-d'))
            ->totalScheduled($airline_id, $service_type_id, $fleet_ids)
            ->get();

        return compact('total', 'overdue');
    }

    /**
     * Display the specified resource.
     *
     * @param int $airline_id
     * @param int $service_type_id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function show(int $airline_id, int $service_type_id, $fleet_ids)
    {
        if ($fleet_ids == 0) {
            $fleet_ids = null;
        } else {
            $fleet_ids = explode(',', $fleet_ids);
        }

        $taskGroups = TaskGroup::query()
            ->with(['tasks'])
            ->where('airline_id', $airline_id)
            ->where('service_type_id', $service_type_id)
            ->orderBy('title')
            ->get();

        $serviceType = ServiceType::query()
            ->find($service_type_id);

        return view('dashboard.tiles-tasks-compliance', [
            'serviceType' => $serviceType,
            'taskGroups' => $taskGroups,
            'compliance' => $this->index($airline_id, $service_type_id, $fleet_ids)
        ]);
    }
}
