<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Services\ServiceTotals;
use App\ServiceLog;
use App\ServiceType;
use App\TaskGroup;
use DB;
use Illuminate\Http\Request;

class AirlineServiceTotalsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @param int $airline_id
     * @param int $service_type_id
     * @param array|null $period
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(int $airline_id, int $service_type_id, ?array $period)
    {
        $totals = ServiceLog::query()
            //->with(['task', 'task.taskGroup'])
            ->select(['task_id', DB::raw('count(task_id) as total')])
            ->where('utp_reason', 0)
            ->where('airline_id', $airline_id)
            ->where('service_type_id', $service_type_id)
            ->whereBetween('start', $period)
            ->groupBy('task_id')
            ->get()
            ->pluck('total', 'task_id');

        return $totals;

        //return response()->json(['data' => $totals], 200);
    }

    /**
     * Display the specified resource.
     *
     * @param $airline_id
     * @param $service_type_id
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\Response|\Illuminate\View\View
     */
    public function show($airline_id, $service_type_id)
    {
        $taskGroups = TaskGroup::query()
            ->with(['tasks'])
            ->where('airline_id', $airline_id)
            ->where('service_type_id', $service_type_id)
            ->get();

        $serviceType = ServiceType::query()
            ->find($service_type_id);

        // 24
        $day = $this->index($airline_id, $service_type_id, [now()->startOfDay(), now()]);

        // week
        $week = $this->index($airline_id, $service_type_id, [now()->startOfWeek(), now()->endOfWeek()]);

        // month
        $month = $this->index($airline_id, $service_type_id, [now()->startOfMonth(), now()->endOfMonth()]);

        // YTD
        $ytd = $this->index($airline_id, $service_type_id, [now()->startOfYear(), now()->endOfYear()]);

        //print_r($totals);

        //return response()->json(['data' => $totals], 200);

        // render a table for this
        return view('dashboard.tiles-tasks', [
            'serviceType' => $serviceType,
            'taskGroups' => $taskGroups,
            'totals' => compact('day', 'week', 'month', 'ytd')
        ]);
    }

    public function service($user_id, $airline_id)
    {
        return view('dashboard.tiles-service', [
            'airlineServicesTotals' => (new ServiceTotals())->airlineServicesTotals($user_id, $airline_id)
        ]);
    }

    public function serviceTasks($airline_id, $service_type_id, $position)
    {
        if ($position == 'sidebar') {
            return view('dashboard.sidebar-totals', [
                'taskTotals' => (new ServiceTotals())->airlineTasksTotals($airline_id, $service_type_id)
            ]);
        }
        return view('dashboard.tiles-tasks', [
            'taskTotals' => (new ServiceTotals())->airlineTasksTotals($airline_id, $service_type_id)
        ]);
    }

}
