<?php

namespace App\Http\Controllers\Api;

use App\Barcode;
use App\Fleet;
use App\Schedule;
use App\User;

use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests\RegisterBarcodeRequest;
use App\Http\Requests\StoreBarcodeRequest;
use App\Transformers\FleetTransformer;
use App\Transformers\BarcodeTransformer;

class BarcodeController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(StoreBarcodeRequest $request)
    {
        $barcode = new Barcode;
        $barcode->barcode = $request->barcode;
        $barcode->station_id = 31;
        $barcode->fleet_id = $request->fleet_id;

        $barcode->save();
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $unit = Barcode::where('barcode', $id)->with('fleet')->first();

        return $unit;
    }

    public function fleetId($id)
    {
        $barcode = Barcode::where('barcode', $id)->with(['fleet' => function ($query) {
            $query->withTrashed();
        }])->first();
        
        if (!empty($barcode)) {
            return fractal()
            ->item($barcode)
            ->transformWith(new BarcodeTransformer)
            ->toArray();
        } else {
            return response()->json(['data' => null], 200);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    public function register(RegisterBarcodeRequest $request)
    {
        $fleet = new Fleet;
        $fleet->tail_number = $request->barcode;
        $fleet->gate_id = $request->get('gate_id', $fleet->gate_id);
        $fleet->airline_id = 136;
        $fleet->aircraft_type_id = $request->get('aircraft_type_id', $fleet->aircraft_type_id);
        $fleet->save();

        $barcode = new Barcode;
        $barcode->barcode = $request->barcode;
        $barcode->station_id = 31;
        $barcode->fleet_id = $fleet->id;
        $barcode->save();

        if ($request->barcode_to_replace > 0) {
            $history = new \App\BarcodeHistory;
            $history->active_fleet_id = $fleet->id;
            $history->replaced_fleet_id = $request->barcode_to_replace;
            $history->save();

            $replecedBarcode = Barcode::where('fleet_id', $request->barcode_to_replace)->first();
            Fleet::destroy($replecedBarcode->fleet_id);
            $replecedBarcode->delete();
        }

        $this->scheduleTasks($fleet->id);

        $fleet = Fleet::findOrFail($fleet->id);
        return fractal()
            ->item($fleet)
            ->transformWith(new FleetTransformer)
            ->toArray();

        return response()->json(['data' => ['fleet' => $fleet]], 200);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function scheduleTasks($fleet_id)
    {
        $tasks = [531, 532, 533, 534];

        foreach ($tasks as $task_id) {
            $this->scheduleTask($fleet_id, $task_id);
        }
    }
    public function scheduleTask($fleet_id, $task_id)
    {
        $user = User::find(Auth::user()->id);
        $schedule = new Schedule;
    
        $schedule->date = '2010-10-10';
        $schedule->user_id = $user->legacy_id;
        $schedule->task_id = $task_id;
        $schedule->tail_number_id = $fleet_id;
        $schedule->reference_id = $fleet_id;
        $schedule->reference_table = 'fleet';
        $schedule->service_type_id = 12;

        $schedule->save();
    }

    public function createFromFleet($key, $id, $start_at_title = null)
    {
        $fleet = Fleet::where($key, $id)->where('tail_number', '>', $start_at_title)->get();
        //dd($fleet);

        if (!empty($fleet)) {
            foreach ($fleet as $ac) {
                $barcode = new Barcode;
                $barcode->barcode = $ac->tail_number;
                $barcode->station_id = 31;
                $barcode->fleet_id = $ac->id;
                $barcode->save();
            }
        }
    }
}
