<?php

namespace App\Http\Controllers\Api;

use App\Fleet;

use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests\UpdateFleetRequest;
use App\Transformers\FleetTransformer;

class FleetController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request, $id)
    {
        $fleet = Fleet::select('id', 'tail_number')->where('airline_id', $id)->get();
        
        return response()->json(['data' => $fleet], 200);
        
        /* too slow with fractal
        $fleet = Fleet::where('airline_id', $id)->get();
        dd(count($fleet));
        return fractal()
            ->collection($fleet)
            ->transformWith(new FleetTransformer)
            ->toArray();
        */
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $fleet = Fleet::find($id); //$fleet = Fleet::withTrashed()->find($id);
        
        return fractal()
            ->item($fleet)
            ->transformWith(new FleetTransformer)
            ->toArray();
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateFleetRequest $request, $id)
    {
        $fleet = Fleet::find($id);
        $fleet->user_id = Auth::user()->id;
        $fleet->gate_id = $request->get('gate_id', $fleet->gate_id);
        $fleet->aircraft_type_id = $request->get('aircraft_type_id', $fleet->aircraft_type_id);

        $fleet->save();

        return response()->json(['data' => ['fleet' => $fleet]], 200);
    }

    /**
     * Undlete the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function undelete(Request $request, $id)
    {
        $unit = Fleet::withTrashed()->find($id);
        $unit->deleted = 0;
        $unit->deleted_at = null;
        $unit->save();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function generate()
    {
        /*
        for ($i = 1000; $i <= 5000; $i++) {
            $fleet = new Fleet;
            $fleet->user_id = 1;
            $fleet->deleted = 0;
            $fleet->airline_id = 134;
            $fleet->tail_number = $i;
            $fleet->aircraft_type_id = 319;
            $fleet->save();
        }
        */
    }
}
