<?php

namespace App\Http\Controllers\Api;

use App\Schedule;
use App\Todo;
use App\Fleet;
use App\Task;
use App\UserMeta;

use App\Http\Controllers\Controller;
use App\Transformers\ScheduleTransformer;
use App\Transformers\ComplianceTransformer;
use Carbon\Carbon;
use League\Fractal\Pagination\IlluminatePaginatorAdapter;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class ScheduleController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)//, $by, $id
    {
        $schedules = Schedule::withFleet($request)
            ->filter($request)
            ->order($request)
            ->paginate(15);
        $schedulesCollection = $schedules->getCollection();
        
        return fractal()
            ->collection($schedulesCollection)
            ->transformWith(new ScheduleTransformer)
            ->paginateWith(new IlluminatePaginatorAdapter($schedules))
            ->toArray();
    }

    public function compliance(Request $request)
    {
        //\DB::enableQueryLog();
        $schedules = Schedule::compliance($request)
            ->filter($request)
            ->order($request)
            ->paginate(15);
        //->get();

        //->paginate(15);
        //$q = \DB::getQueryLog();
        //print_r($q);
        
        $schedulesCollection = $schedules->getCollection();
        //$schedulesCollection = $schedules->sortBy('barcode');

        return fractal()
            ->collection($schedulesCollection)
            ->transformWith(new ComplianceTransformer)
            ->paginateWith(new IlluminatePaginatorAdapter($schedules))
            ->toArray();
    }

    public function complianceAll(Request $request)
    {
        $schedulesCollection = Schedule::compliance($request)
            ->filter($request)
            ->order($request)
            ->get();

        return fractal()
            ->collection($schedulesCollection)
            ->transformWith(new ComplianceTransformer)
            ->toArray();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $schedule = Schedule::find($id);

        //dd($schedule);

        return fractal()
            ->item($schedule)
            ->transformWith(new ScheduleTransformer)
            ->toArray();
    }

    public function byFleet($id)
    {
        $userTaskPermissions = UserMeta::taskPermissions();

        if (is_array($userTaskPermissions)) {
            $schedules = Schedule::where('tail_number_id', $id)
                ->whereIn('task_id', $userTaskPermissions)
                ->get();
        } else {
            $schedules = Schedule::where('tail_number_id', $id)
                ->get();
        }
        
        return fractal()
            ->collection($schedules)
            ->transformWith(new ScheduleTransformer)
            ->toArray();
    }

    public function byFleetTask($fleet_id, $task_id)
    {
        $schedule = Schedule::where('tail_number_id', $fleet_id)->where('task_id', $task_id)->first();

        return fractal()
            ->item($schedule)
            ->transformWith(new ScheduleTransformer)
            ->toArray();
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $schedule = Schedule::find($id);
        $schedule->date = $request->date;
        $schedule->save();

        return fractal()
            ->item($schedule)
            ->transformWith(new ScheduleTransformer)
            ->toArray();
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function cycle(Request $request, $id)
    {
        $schedule = Schedule::find($id);
        $proposed = $this->nextCycleDate($schedule);

        $schedule->user_id = Auth::user()->id;
        $schedule->date = $proposed;

        $schedule->save();

        return fractal()
            ->item($schedule)
            ->transformWith(new ScheduleTransformer)
            ->toArray();
    }

    private function nextCycleDate($schedule)
    {
        $scheduled = $schedule->date;
        $cycle = $schedule->task->cycle;

        if ($schedule->task_id == 534) { //Audit
            $proposed = $this->cycleFirstOfNextMonth();
        } elseif ($schedule->task_id == 531) { // Clean
            $proposed = $this->cycleTwicePerYear();
        } else {
            // update to use the STOP time
            $proposed = date('Y-m-d', strtotime(' + ' . $cycle . ' days'));
        }

        return $proposed;
    }

    private function cycleFirstOfNextMonth()
    {
        $proposed = new Carbon('first day of next month');
        $proposed = $proposed->format('Y-m-d');

        return $proposed;
    }

    private function cycleTwicePerYear()
    {
        $now = Carbon::parse();
        //echo $now;
        //echo Carbon::parse('first day of July');
        //echo Carbon::parse('last day of December')->addSeconds(86399);
        //echo Carbon::parse('first day of January');
        //echo Carbon::parse('last day of June')->addSeconds(86399);

        if (Carbon::parse('first day of January')->lessThanOrEqualTo($now) && Carbon::parse('last day of June')->addSeconds(86399)->greaterThanOrEqualTo($now)) {
            $proposed = Carbon::parse('first day of July');
        } elseif (Carbon::parse('first day of July')->lessThanOrEqualTo($now) && Carbon::parse('last day of December')->addSeconds(86399)->greaterThanOrEqualTo($now)) {
            $proposed = Carbon::parse('first day of January next year');
        } else {
        }

        $proposed = $proposed->format('Y-m-d');
        return $proposed;
    }

    public function asCsv(Request $request)
    {
        ini_set('max_execution_time', 600);

        $schedules = Schedule::withFleet($request)
            ->filter($request)
            ->order($request)
            ->get();
        
        header('Content-Disposition: attachment; filename="terminal-schedules-' . time() . '.csv"');
        header("Cache-Control: private");
        header("Content-Type: application/octet-stream");
        header("Content-Transfer-Encoding: binary\n");

        echo "Scheduled,POD#,Type,Task,Gate\n";
        foreach ($schedules as $schedule) {
            echo $schedule->date . ',';
            echo '"' . $schedule->fleet->tail_number . '",';
            echo $schedule->aircraft . ',';
            echo $schedule->task->title . ',';
            echo $schedule->fleet->gate->title . "\n";
        }
        exit;
    }

    public function generate()
    {
        $fleet = Fleet::where('airline_id', 136)->get();
        foreach ($fleet as $unit) {
            $this->scheduleTasks($unit->id);
        }
    }

    public function scheduleTasks($fleet_id)
    {
        $schedule = new Schedule;
        //$tasks = Task::where('service_type_id', 12)->get();
    
        $schedule->date = '2010-10-10';
        $schedule->user_id = 1;
        $schedule->tail_number_id = $fleet_id;
        $schedule->reference_id = $fleet_id;
        $schedule->reference_table = 'fleet';
        $schedule->service_type_id = 12;

        $schedule->task_id = 534;
        //$schedule->save();
    }
}
