<?php

namespace App\Http\Controllers\Api;

use App\Todo;
use App\Schedule;
use App\UserMeta;

use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Request;
use App\Transformers\ScheduleTransformer;
use League\Fractal\Pagination\IlluminatePaginatorAdapter;
use App\Http\Controllers\Controller;
use App\Http\Requests\StoreTodoRequest;

class TodoController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $userTaskPermissions = UserMeta::taskPermissions();

        if (is_array($userTaskPermissions)) {
            $schedules = Schedule::where('service_type_id', 12)
                ->whereIn('schedules.task_id', $userTaskPermissions)
                ->has('todo')
                ->has('fleet')
                ->get();
        } else {
            $schedules = Schedule::where('service_type_id', 12)->has('todo')->has('fleet')->get();
        }

        return fractal()
            ->collection($schedules)
            ->transformWith(new ScheduleTransformer)
            ->toArray();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(StoreTodoRequest $request)
    {
        $todo = new Todo;
        $todo->schedule_id = $request->schedule_id;
        $todo->user_id = Auth::user()->id;

        $todo->save();

        return response()->json(['data' => ['id' => $todo->id]], 200);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $todo = new Todo;
        $todo->findOrFail($id)->delete();
    }
}
