<?php

namespace App\Http\Controllers;

use App\Airline;
use App\Employee;
use App\Mail\UltiproImportDetails;
use App\Schedule;
use App\Station;
use App\TasksByJob;
use App\UltiproImport;
use App\UserLegacy;
use App\UserMeta;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;

class EmployeeController extends Controller
{
    public function __construct()
    {
        $this->todaysFile = 'UP_'. date('Ymd') .'.csv';
        //$this->todaysFile = 'UP_'. date('Ymd', strtotime('-1 days')) .'.csv';
        $this->stations = Station::all()->pluck('id', 'abbr');
        $this->airlineIdByStationId = Airline::whereNotNull('station_id')->get()->pluck('id', 'station_id');

        $this->trainingTasks = TasksByJob::getTasks('training');
        $this->operationsTasks = TasksByJob::getTasks('operations');

        $this->thresholdInHours = 1;
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Post Ultipro import to add/update/disable employees.
     *
     * @return \Illuminate\Http\Response
     */
    public function postImport()
    {
        $activated = [];
        $terminated = [];
        $updated = [];
        $created = [];
        $now = new Carbon;

        $import = UltiproImport::with('employee')
            ->where('file', $this->todaysFile)
            ->get()
            ->groupBy('employee_number');

        if ($import->isNotEmpty()) {

            foreach($import as $item) {
                $item = $item->first();

                if (empty($item->employee)) {
                    $employeeNameMatch = self::employeeNameMatch($item);

                    if ($employeeNameMatch['match'] === true) {
                        echo $item->employee_name . ' got a match.<br>';
                        self::updateFromImport($item, $employeeNameMatch['updateName']);
                    } else{ //if ($item['term_date'] != 'Terminated')
                        echo '<strong>' . $item->employee_name . ' not found. create.</strong><br>';
                        self::createFromImport($item);
                    }
                }

                if (! empty($item->employee)) {
                    //dd($item->employee);
                    self::updateFromImport($item, false);
                }

                self::activate($item);
                self::terminated($item);
            }

            $updated = self::employeeAudit('updated');
            $created = self::employeeAudit('created');
            $terminated = self::employeeAudit('terminated');

            $data = [
                'fileUrl' => url()->to('/ultipro/file/'. $this->todaysFile .'?h=' . Hash::make($this->todaysFile)),
                'fileName' => $this->todaysFile,
                'terminated' => $terminated,
                'updated' => $updated,
                'created' => $created,
                'overtime' => self::overtime(),
                'mia' => self::mia(),
                'invalid' => self::invalid(),
                'trainingTasks' => $this->trainingTasks,
                'operationsTasks' => $this->operationsTasks,
            ];

            $stats = self::importDetails();
            $data += $stats;

            Log::notice('Post import done.');

            self::mail('tvanwinkle.eng@accufleet.com', $data);
            self::mail('dhopkins@sailingstones.com', $data);

            Log::notice('Unfiltered email sent.');

            self::reportMailPerUser();

            Log::notice('All per user emails sent.');

            echo 'done';

        } else {
            Log::notice('No records to post process for today.');
        }
    }

    public function reportMailPerUser()
    {
        $emalUser = UserMeta::with('legacy')
            ->whereNotNull('reports')
            ->get();

        foreach($emalUser as $user) {
            if (is_array($user->reports) && in_array('ultipro', $user->reports) && ! empty($user->stations)) {
                self::reportMailPerStations($user);
            }
        }
        Log::notice('Mail loop per user done.');
        echo 'done';
    }

    public function reportMailPerStations($user)
    {
        $stationAbbrs = [];
        $updated = self::employeeAudit('updated', $user->stations);
        $created = self::employeeAudit('created', $user->stations);
        $terminated = self::employeeAudit('terminated', $user->stations);

        if ($user->stations !== null) {
            $stationAbbrs = Station::whereIn('id', $user->stations)
                ->pluck('abbr', 'id');
        }

        $data = [
            'stationAbbrs' => $stationAbbrs,
            'fileUrl' => url()->to('/ultipro/file/'. $this->todaysFile .'?h=' . Hash::make($this->todaysFile)),
            'fileName' => $this->todaysFile,
            'terminated' => $terminated,
            'updated' => $updated,
            'created' => $created,
            'overtime' => self::overtime($stationAbbrs),
            'mia' => self::mia($user->stations, $stationAbbrs),
            'invalid' => self::invalid($user->stations),
            'trainingTasks' => $this->trainingTasks,
            'operationsTasks' => $this->operationsTasks,
        ];

        $stats = self::importDetails();
        $data += $stats;

        self::mail($user->legacy->email, $data);
        Log::notice('Mailed to ' . $user->legacy->email);
    }

    public function mail($to, $data)
    {
        if (app()->isLocal()) {
            Mail::to($to)
                ->send(new UltiproImportDetails($data));
        } else {
            Mail::to($to)
                //->bcc('dhopkins@sailingstones.com')
                ->send(new UltiproImportDetails($data));
        }
        Log::notice('Mail sent to ' . $to);
    }

    public function mailTest()
    {
        $user = UserMeta::with('legacy')
            ->findOrfail(1);
        self::reportMailPerStations($user);

        Log::notice('Mail sent ONLY to ME');
    }
    public function employeeAudit($method, $stationIds = null)
    {
        $employees = [];
        if ($method == 'created') {
            $now = new Carbon;
            $employees = Employee::with('station')
                ->where('created_at', '>=', $now->subHour($this->thresholdInHours)->toDateTimeString())
                ->when($stationIds !== null, function ($q) use ($stationIds) {
                    return $q->whereIn('station_id', $stationIds);
                })
                ->get();
        } elseif ($method == 'updated') {
            $now = new Carbon;
            $employees = Employee::with('station')
                ->where('created_at', '<=', $now->toDateTimeString())
                ->where('updated_at', '>=', $now->subHour($this->thresholdInHours)->toDateTimeString())
                ->when($stationIds !== null, function ($q) use ($stationIds) {
                    return $q->whereIn('station_id', $stationIds);
                })
                ->get();
        } elseif ($method == 'terminated') {
            $now = new Carbon;
            $employees = Employee::with('station')
                ->withTrashed()
                ->where('deleted_at', '<=', $now->toDateTimeString())
                ->where('deleted_at', '>=', $now->subHour($this->thresholdInHours)->toDateTimeString())
                ->when($stationIds !== null, function ($q) use ($stationIds) {
                    return $q->whereIn('station_id', $stationIds);
                })
                ->get();
        }

        if (! empty($employees)) {
            $employees->each(function ($item, $key) {
                if (isset($this->operationsTasks[$item->job]) && isset($this->operationsTasks[$item->job][$item->station->id])) {
                    $item->operations_tasks = $this->operationsTasks[$item->job][$item->station->id];
                } else {
                    $item->operations_tasks = [];
                }
                if (isset($this->trainingTasks[$item->job]) && isset($this->trainingTasks[$item->job][$item->station->id])) {
                    $item->training_tasks = $this->trainingTasks[$item->job][$item->station->id];
                } else {
                    $item->training_tasks = [];
                }
                return $item;
            });
        }

        return $employees;
    }

    public function createFromImport($item)
    {
        $uniqueEmployeeNumber = Employee::where('number', $item->employee_number)->first();
        //dd($uniqueEmployeeNumber);
        if ($uniqueEmployeeNumber !== null) {
            $employeeNumber = rand(11000000,11999999);
        } else {
            $employeeNumber = $item->employee_number;
        }
        $employee = new Employee;
        $station_id = $this->stations[$item['station']];
        $airline_id = $this->airlineIdByStationId[$station_id];

        $employee->name = $item->employee_name;
        $employee->number = $employeeNumber;
        $employee->job = $item->job;
        $employee->doh = $item->hire_date;
        $employee->rate = $item->rate;
        $employee->airline_ids = json_encode([132, $airline_id]);
        $employee->station_id = $station_id;
        $employee->save();
        return $item;
    }
    public function updateFromImport($item, $updateName = false)
    {
        $employee = $item->employee;
        $station_id = $this->stations[$item['station']];
        $airline_id = $this->airlineIdByStationId[$station_id];

        if ($updateName == true) {
            $employee->name = $item->employee_name;
        }

        $employee->number = $item->employee_number;
        $employee->doh = $item->hire_date;
        $employee->job = $item->job;
        $employee->jobs = self::syncJobs($employee, $item);
        $employee->rate = $item->rate;
        $employee->airline_ids = json_encode([132, $airline_id]);
        $employee->station_id = $station_id;
        $employee->save();
        return $item;
    }

    public function syncJobs($employee, $item)
    {
        if (empty($item->job_desc)) {
            return null;
        }

        if (empty($employee->job_desc) && $employee->job != $item->job_desc) {
            return json_encode([$item->job_desc]);
        } elseif ($employee->job_desc != null && ! in_array($employee->job_desc, json_decode($employee->job_desc, true))) {
            $jobs = json_decode($employee->job_desc);
            $jobs[] = $employee->job_desc;
            return json_encode($jobs);
        }

    }

    public function activate($item)
    {
        $employee = $item->employee;
        if ($item->employee !== null && $item->term_date === null && $item->status != 'Terminated') {
            $employee->deleted_at = null;
            $employee->deleted = 0;
            $employee->save();
            return $item;
        }
        return false;
    }

    public function terminated($item)
    {
        $employee = $item->employee;

        if ($item->employee !== null && $item->term_date !== null && $item->status == 'Terminated' && $employee->deleted == 0) {
            $employee->term_date = $item->term_date;
            $employee->deleted = 1;
            $employee->number = 99 . rand(100,999) . $employee->number;
            $employee->save();
            $employee->delete();

            if ($employee->user_id > 0) {
                $user = UserLegacy::where('id', $employee->user_id)
                    ->where('active', 1)
                    ->first();
                if (! empty($user)) {
                    $user->username = $user->username . '-deleted';
                    $user->email = $user->email . '-deleted';
                    $user->active = 0;
                    $user->save();
                }
            }
            return $item;
        }
        return false;
    }

    public function terminateAll()
    {
        $terminate = UltiproImport::where('status', 'Terminated')
            ->where('term_date', '>', date('Y-m-d', strtotime(' - 7 days')))
            ->with('employee')
            ->distinct()
            ->get();
        //dd($terminate);

        foreach($terminate as $item) {
            self::terminated($item);
        }
        echo 'done.';
    }

    public function overtime($stationAbbrs = null)
    {
        return UltiproImport::where('file', $this->todaysFile)
            ->where('payident', '!=', 'REG')
            ->when($stationAbbrs !== null, function ($q) use ($stationAbbrs) {
                return $q->whereIn('station', $stationAbbrs);
            })
            ->get();
    }

    public function mia($stationIds = null, $stationAbbrs = null)
    {
        $employees = Employee::select(['employees.number', 'employees.name', 'stations.abbr'])
            ->join('stations', 'employees.station_id', '=', 'stations.id')
            ->where('salaried', 0)
            ->when($stationIds !== null, function ($q) use ($stationIds) {
                return $q->whereIn('station_id', $stationIds);
            })
            ->orderBy('stations.abbr')
            ->orderBy('name')
            ->get()
            ->keyBy('number')
            ->toArray();

        $batch = UltiproImport::where('workdate', '<=', date('Y-m-d'))
            ->when($stationAbbrs !== null, function ($q) use ($stationAbbrs) {
                return $q->whereIn('station', $stationAbbrs);
            })
            ->where('workdate', '>=', date('Y-m-d', strtotime(' - 4 days')))
            ->get()
            ->pluck('employee_name', 'employee_number')
            ->toArray();

        $lastWorked = UltiproImport::where('workdate', '<=', date('Y-m-d', strtotime(' - 4 days')))
            ->orderBy('workdate')
            ->get()
            ->pluck('workdate', 'employee_number')
            ->toArray();

        $mia = array_diff_key($employees, $batch);

        $miaWithLastWorked = collect($mia)->map(function ($item, $key) use($lastWorked) {
            if (isset($lastWorked[$item['number']])) {
                return array_merge($item, ['lastworked' => $lastWorked[$item['number']]]);
            } else {
                return null;
            }
        })->reject(function ($value, $key) {
            return empty($value);
        });

        return $miaWithLastWorked;
    }

    public function invalid($stationIds = null)
    {
        return Employee::with(['station'])
            ->when($stationIds !== null, function ($q) use ($stationIds) {
                return $q->whereIn('station_id', $stationIds);
            })
            ->where('number', '>', 999999)
            ->where('number', '<', 11999999)
            ->orderBy('name')
            ->orderBy('name')
            ->get();
    }

    public function employeeNameMatch($item)
    {
        $importEmployeeName = $item->employee_name;

        if (isset($item->employee->name)) {
            $name = $item->employee->name;

            if ($importEmployeeName == $name) {
                return ['match' => true, 'updateName' => false];
            } elseif (strtolower($importEmployeeName) == strtolower($name)) {
                return ['match' => true, 'updateName' => true, 'name' => $importEmployeeName];
            } elseif (self::searchByName($importEmployeeName, $name) === true) {
                return ['match' => true, 'updateName' => true, 'name' => $importEmployeeName];
            }
        } else {
            $employee = self::findEmployeeByName($importEmployeeName);
            if ($employee !== false) {
                return ['match' => false, 'updateName' => false, 'foundEmployee' => $employee];
            }

            $employee = self::findEmployee($importEmployeeName);
            if ($employee !== false) {
                return ['match' => false, 'updateName' => true, 'foundEmployee' => $employee];
            }
        }
    }

    public function searchByName($importEmployeeName, $name)
    {
        $nameParts = $this->nameParts($importEmployeeName);

        if (strpos($name, $nameParts['lastName']) > -1 && strpos($name, $nameParts['firstName']) > -1) {
            return true;
        }
    }

    public function findEmployeeByName($name)
    {
        $employees = Employee::where('name', $name)
            ->get();

        if (count($employees) == 1) {
            return $employees->first();
        } elseif (count($employees) > 1) {
            Log::warning('I have to many emplyees returned. findEmployeeByName');
            dd($employees);
        } else {
            return false;
        }
    }

    public function findEmployee($importEmployeeName)
    {
        $nameParts = $this->nameParts($importEmployeeName);

        $employees = Employee::where('name', 'like', '%' . $nameParts['lastName'] . '%')
            ->where('name', 'like', '%' . $nameParts['firstName'] . '%')
            ->get();

        if (count($employees) == 1) {
            return $employees->first();
        } elseif (count($employees) > 1) {
            Log::warning('I have to many emplyees returned. findEmployee');
            dd($employees);
        }
    }

    public function nameParts($importEmployeeName)
    {
        $importNameParts = explode(', ', $importEmployeeName);
        $importLastName = $importNameParts[0];
        $importFirstName = $importNameParts[1];
        //$importLastName = explode(' ', $importNameParts[0]);
        //$importLastName = $importLastName[0];
        //$importFirstName = explode(' ', $importNameParts[1]);
        //$importFirstName = $importFirstName[0];

        return [
            'lastName' => $importLastName,
            'firstName' => $importFirstName,
        ];
    }

    public function importDetails()
    {
        $import = UltiproImport::with('employee')
            ->where('file', $this->todaysFile)
            ->get();
        $stats['fileName'] = $this->todaysFile;
        $stats['initiated'] = $import->first()->import_initiated;
        $stats['total'] = $import->count();
        $stats['totalGrouped'] = $import->groupBy('employee_number')->count();

        return $stats;
    }

    public function employeeSchedules()
    {
        $now = new Carbon;

        $employees = Employee::where('updated_at', '>=', $now->subHour($this->thresholdInHours)->toDateTimeString())->get();

        //$employees = Employee::where('id', 1221)->get();

        foreach($employees as $employee) {

            foreach($employee->allJobs as $jobs) {
                if (isset($this->trainingTasks[$jobs][$employee->station_id])) {
                    $trainingIds = array_keys($this->trainingTasks[$jobs][$employee->station_id]);
                } else {
                    $trainingIds = [];
                }

                if (isset($this->operationsTasks[$jobs][$employee->station_id])) {
                    $operationsTasks = array_keys($this->operationsTasks[$jobs][$employee->station_id]);
                } else {
                    $operationsTasks = [];
                }

                if ( ! empty($trainingIds)) {
                    self::setEmployeeTaskSchedules(8, $trainingIds, $employee);
                }
                if ( ! empty($operationsTasks)) {
                    self::setEmployeeTaskSchedules(10, $operationsTasks, $employee);
                }
            }
        }

        Log::notice('Schedules created.');
        echo 'schedules created.';
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function deactivateUsersNotInCSV()
    {
        $csv = 'uploads/import-employees-20200120.csv';

        // create collection from import
        $file = fopen(storage_path('/app/'. $csv), 'r');
        $rows = [];

        while ( ($data = fgetcsv($file, 0, ',')) !== false ) {
            $rows[] = $data;
        }

        $header = $rows[0];
        unset($rows[0]);
        $importedEmployees = [];

        foreach($rows as $row) {
            $employeeNumber = (int) $row[2];
            $importedEmployees[$employeeNumber] = array_combine($header, $row);
        }

        ksort($importedEmployees);

        //dd($importedEmployees);

        //get all employees indexed by employee id
        $employees = Employee::withTrashed()
            ->where('updated_at', '<', '2020-01-20 00:00:00')
            ->where('deleted', 0)
            ->whereNull('deleted_at')
            ->get()
            ->keyBy('number')
            ->toArray();

        ksort($employees);

        //dd($employees);

        $unmatchedEmployeeNumbers = array_diff_key($importedEmployees, $employees);
        dd($unmatchedEmployeeNumbers);

        foreach($unmatchedEmployeeNumbers as $employee) {
            //dd($employee);
            $deleteEmployee = Employee::withTrashed()->findOrfail($employee['id']);
            //dd($deleteEmployee);
            $deleteEmployee->deleted = 1;
            $deleteEmployee->save();
            //$deleteEmployee->delete();
        }

        echo '..';
    }

    public function searchByNameInCSV()
    {
        $csv = 'uploads/import-employees-20200120.csv';

        // create collection from import
        $file = fopen(storage_path('/app/'. $csv), 'r');
        $rows = [];

        while ( ($data = fgetcsv($file, 0, ',')) !== false ) {
            $rows[] = $data;
        }

        $header = $rows[0];
        unset($rows[0]);
        $importedEmployees = [];

        foreach($rows as $row) {
            $employeeName = $row[0];
            $importedEmployees[$employeeName] = array_combine($header, $row);
        }

        ksort($importedEmployees);

        //dd($importedEmployees);

        //get all employees indexed by employee id
        $employees = Employee::withTrashed()
            ->where('created_at', '<', '2020-01-20 00:00:00')
            ->where('deleted', 1)
            ->get()
            ->keyBy('name')
            ->toArray();

        ksort($employees);

        //dd($employees);

        $matchedEmployeeNames = array_intersect_key($employees, $importedEmployees);
        //dd($matchedEmployeeNames);

        foreach($matchedEmployeeNames as $employee) {
            //dd($employee);
            $undeleteEmployee = Employee::withTrashed()->findOrfail($employee['id']);
            //dd($undeleteEmployee);
            $undeleteEmployee->deleted = 0;
            $undeleteEmployee->deleted_at = null;
            $undeleteEmployee->save();
        }

        echo '..';
    }

    public function freeEmployeeNumberOfDeleted()
    {
        $employees = Employee::withTrashed()
            ->where('deleted', 1)
            ->whereNotNull('deleted_at')
            ->get();

        foreach($employees as $employee) {
            $employee->number = 99 . rand(100000,999999);
            $employee->save();
        }

        Log::notice('Emolyee number freed ' . $employee->number);
        echo '.';
    }

    public function fixDeleted()
    {
        $employees = Employee::withTrashed()
            ->whereNotNull('deleted_at')
            ->get();

        foreach($employees as $employee) {
            $employee->deleted = 1;
            $employee->save();
        }
        echo '.';
    }

    public function fixNullEmployeeNumber()
    {
        $employees = Employee::withTrashed()
            ->whereNull('number')
            ->get();

        foreach($employees as $employee) {
            $employee->number = 99 . rand(100000,999999);
            $employee->deleted = 1;
            $employee->save();
        }
        echo '.';
    }

    public static function setEmployeeTaskSchedules(int $serviceTypeID, array $taskIds, object $employee)
    {
        foreach($taskIds as $taskId) {
            $schedule = Schedule::where('reference_table', 'employees')
                ->where('reference_id', $employee->id)
                ->where('task_id', $taskId)
                ->orderBy('date', 'desc')
                ->get();
            //dd($schedule);
            if ($schedule->isEmpty()) {
                //dd('create');
                $now = new Carbon;
                Schedule::create([
                    'reference_table' => 'employees',
                    'reference_id' => $employee->id,
                    'task_id' => $taskId,
                    'service_type_id' => $serviceTypeID,
                    'date' => $now->addDay(5)->toDateTimeString(),
                    'user_id' => 99999,
                ]);
            } else {
                if ($schedule->count() > 1) {
                    $schedule->shift();
                    //dd('delete all (oldest) but the frist');
                    foreach($schedule as $s) {
                        $s->destroy($s->id);
                    }
                }
            }
        }
    }

    /*
    public function operationJobs($item)
    {
        $operationJobs = TasksByJob::where('type', 'operations')
            ->where('station_id', $this->stations[$item->station])
            ->where('job', $item->job)
            ->get();

        if ($operationJobs->isNotEmpty()) {
            $item->operation = $operationJobs;
        } else {
            $item->operation = 'No tasks to add for this job at this station.';
        }
        return $item;
    }
    */
}
