<?php

namespace App\Http\Controllers;

use App\EmployeeTest;
use App\Mail\UltiproImportDetails;
use App\UltiproImport;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;

class EmployeeTestController extends Controller
{
    public function __construct()
    {
        $this->todaysFile = 'UP_'. date('Ymd', strtotime('-5 days')) .'.csv';
        //$this->todaysFile = 'UP_'. date('Ymd') .'.csv'
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Post Ultipro import to add/update/disable employees.
     *
     * @return \Illuminate\Http\Response
     */
    public function postImport()
    {
        $activated = [];
        $terminated = [];
        $updated = [];
        $created = [];
        $now = new Carbon;

        $import = UltiproImport::with('employee')->where('file', $this->todaysFile)->get()->groupBy('employee_number');

        //$import->each(function($item) {
        foreach($import as $item) {
            $item = $item->first();
            $employeeNameMatch = self::employeeNameMatch($item);

            if ($employeeNameMatch['match'] === true) {
                echo $item->employee_name . ' got a match.<br>';
                self::updateFromImport($item, $employeeNameMatch['updateName']);
            } elseif (isset($employeeNameMatch['foundEmployee']) && $employeeNameMatch['foundEmployee'] !== false) {
                $item->employee = $employeeNameMatch['foundEmployee'];
                //print_r($item->employee);
                echo $item->employee_name . ' <strong>deeper search match.</strong><br>';
                self::updateFromImport($item, $employeeNameMatch['updateName']);
            } else {
                echo '<strong>' . $item->employee_name . ' not found. create.</strong><br>';
                self::createFromImport($item);
            }

            if (self::activate($item) !== false) {
                $activated[] = self::activate($item);
            }
            
            if (self::terminated($item) !== false) {
                $terminated[] = self::terminated($item);
            }
        };

        // get updated from the database. This will return all.
        $updated = EmployeeTest::where('created_at', '<=', $now->subHour(1)->toDateTimeString())
            ->where('updated_at', '>=', $now->subHour(1)->toDateTimeString())
            ->get();

        $created = EmployeeTest::where('created_at', '>=', $now->subHour(1)->toDateTimeString())
            ->get();

        $data = [
            'fileName' => $this->todaysFile,
            'activated' => $activated,
            'terminated' => $terminated,
            'updated' => $updated,
            'created' => $created,
        ];

        $stats = self::importDetails();
        $data += $stats;

        self::mail($data);

        //dd($data);
        echo 'done';
    }

    public function createFromImport($item)
    {
        $employee = new EmployeeTest;
        $station = \App\Station::where('abbr', $item['station'])
            ->get()
            ->first();
        $airline = \App\Airline::where('title', 'like', '%' . $item['station'] . ' Operator Check')
            ->get()
            ->first();

        $employee->name = $item->employee_name;
        $employee->number = $item->employee_number;
        $employee->job = $item->job;
        $employee->doh = $item->hire_date;
        $employee->airline_ids = json_encode([132, $airline->id]);
        $employee->station_id = $station->id;
        $employee->save();
        return $item;
    }

    public function updateFromImport($item, $updateName = false)
    {
        $employee = $item->employee;
        $station = \App\Station::where('abbr', $item['station'])
            ->get()
            ->first();
        $airline = \App\Airline::where('title', 'like', '%' . $item['station'] . ' Operator Check')
            ->get()
            ->first();

        if ($updateName == true) {
            $employee->name = $item->employee_name;
        }

        $employee->number = $item->employee_number;
        $employee->doh = $item->hire_date;
        $employee->job = $item->job;
        $employee->airline_ids = json_encode([132, $airline->id]);
        $employee->station_id = $station->id;
        $employee->save();
        return $item;
    }

    public function activate($item)
    {
        $employee = $item->employee;
        if ($item->term_date === null && isset($employee->deleted_at) && $employee->deleted_at !== null) {
            $employee->deleted_at = null;
            $employee->deleted = 0;
            $employee->save();
            return $item;
        }
        return false;
    }

    public function terminated($item)
    {
        $employee = $item->employee;
        if ($item->term_date === null && isset($employee->deleted_at) && $employee->deleted_at !== null) {
            $employee->deleted_at = $item->employee->term_date;
            $employee->deleted = 1;
            $employee->save();
            return $item;
        }
        return false;
    }

    public function employeeNameMatch($item)
    {
        $importEmployeeName = $item->employee_name;

        if (isset($item->employee->name)) {
            $name = $item->employee->name;

            if ($importEmployeeName == $name) {
                return ['match' => true, 'updateName' => false];
            } elseif (strtolower($importEmployeeName) == strtolower($name)) {
                return ['match' => true, 'updateName' => true, 'name' => $importEmployeeName];
            } elseif (self::searchByName($importEmployeeName, $name) === true) {
                return ['match' => true, 'updateName' => true, 'name' => $importEmployeeName];
            }
        } else {
            $employee = self::findEmployeeByName($importEmployeeName);
            if ($employee !== false) {
                return ['match' => false, 'updateName' => false, 'foundEmployee' => $employee];
            }
            
            $employee = self::findEmployee($importEmployeeName);
            if ($employee !== false) {
                return ['match' => false, 'updateName' => true, 'foundEmployee' => $employee];
            }
        }
    }

    public function searchByName($importEmployeeName, $name)
    {
        $nameParts = $this->nameParts($importEmployeeName);

        if (strpos($name, $nameParts['lastName']) > -1 && strpos($name, $nameParts['firstName']) > -1) {
            return true;
        }
    }

    public function findEmployeeByName($name)
    {
        $employees = \App\EmployeeTest::where('name', $name)
            ->get();

        if (count($employees) == 1) {
            return $employees->first();
        } elseif (count($employees) > 1) {
            dd($employees);
        } else {
            return false;
        }
    }

    public function findEmployee($importEmployeeName)
    {
        $nameParts = $this->nameParts($importEmployeeName);

        $employees = \App\EmployeeTest::where('name', 'like', '%' . $nameParts['lastName'] . '%')
            ->where('name', 'like', '%' . $nameParts['firstName'] . '%')
            ->get();

        if (count($employees) == 1) {
            return $employees->first();
        } elseif (count($employees) > 1) {
            dd($employees);
        }
    }

    public function nameParts($importEmployeeName)
    {
        $importNameParts = explode(', ', $importEmployeeName);
        $importLastName = explode(' ', $importNameParts[0]);
        $importLastName = $importLastName[0];
        $importFirstName = explode(' ', $importNameParts[1]);
        $importFirstName = $importFirstName[0];

        return [
            'lastName' => $importLastName,
            'firstName' => $importFirstName,
        ];
    }

    public function importDetails()
    {
        $import = UltiproImport::with('userMeta')
            ->where('file', $this->todaysFile)
            ->get();

        $stats['fileName'] = $this->todaysFile;
        $stats['initiated'] = $import->first()->import_initiated;
        $stats['total'] = $import->count();
        $stats['totalGrouped'] = $import->groupBy('employee_number')->count();

        return $stats;
    }

    public function mail($data)
    {
        //$details = self::importDetails($this->todaysFile);
        Mail::to('dhopkins@sailingstones.com')
            ->send(new UltiproImportDetails($data));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
