<?php

namespace App\Http\Controllers\Services;

use App\Airline;
use App\Http\Controllers\Controller;
use App\ServiceLog;
use App\ServiceType;
use App\TaskGroup;
use App\UserMeta;
use DB;

class ServiceTotals extends Controller
{
    public function getTaskTotals(int $airline_id, int $service_type_id, ?array $period): \Illuminate\Support\Collection
    {
        return ServiceLog::query()
            ->select(['task_id', DB::raw('count(task_id) as total')])
            ->where('deleted', 0)
            ->where('utp_reason', 0)
            ->where('stop', '!=', '0000-00-00 00:00:00')
            ->where('airline_id', $airline_id)
            ->where('service_type_id', $service_type_id)
            ->whereBetween('start', $period)
            ->groupBy('task_id')
            ->get()
            ->pluck('total', 'task_id');
    }

    public function getServiceTotals(int $airline_id, array $serviceTypes, ?array $period): \Illuminate\Support\Collection
    {
        return ServiceLog::query()
            ->select(['service_type_id', DB::raw('count(service_type_id) as total')])
            ->where('deleted', 0)
            ->where('utp_reason', 0)
            ->where('stop', '!=', '0000-00-00 00:00:00')
            ->where('airline_id', $airline_id)
            ->whereBetween('start', $period)
            ->whereIn('service_type_id', $serviceTypes)
            ->groupBy('service_type_id')
            ->get()
            ->pluck('total', 'service_type_id');
    }

    public function airlineServicesTotals($user_id, $airline_id): array
    {
        $airline = Airline::query()->find($airline_id);
        $airlineServiceTypes = json_decode($airline->service_types);
        $userServiceTypes = json_decode(UserMeta::find(['user_id', $user_id])->pluck('service_types')->first());

        $serviceTypes = array_intersect($userServiceTypes, $airlineServiceTypes);

        //$day = $this->getServiceTotals($airline_id, $serviceTypes, [now()->subDay(), now()]);

        //$week = $this->getServiceTotals($airline_id, $serviceTypes, [now()->startOfWeek(), now()->endOfWeek()]);

        $month = $this->getServiceTotals($airline_id, $serviceTypes, [now()->startOfMonth(), now()->endOfMonth()]);

        $ytd = $this->getServiceTotals($airline_id, $serviceTypes, [now()->startOfYear(), now()->endOfYear()]);

        return [
            'serviceTypes' => ServiceType::query()->whereIn('id', $serviceTypes)->orderBy('title')->get(),
            'totals' => compact('month', 'ytd')
        ];
    }

    public function airlineTasksTotals($airline_id, $service_type_id): array
    {
        $taskGroups = TaskGroup::query()
            ->with(['tasks'])
            ->where('airline_id', $airline_id)
            ->where('service_type_id', $service_type_id)
            ->orderBy('title')
            ->get();

        $serviceType = ServiceType::query()
            ->find($service_type_id);

        $day = $this->getTaskTotals($airline_id, $service_type_id, [now()->subDay(), now()]);

        $week = $this->getTaskTotals($airline_id, $service_type_id, [now()->startOfWeek(), now()->endOfWeek()]);

        $month = $this->getTaskTotals($airline_id, $service_type_id, [now()->startOfMonth(), now()->endOfMonth()]);

        $ytd = $this->getTaskTotals($airline_id, $service_type_id, [now()->startOfYear(), now()->endOfYear()]);

        return [
            'serviceType' => $serviceType,
            'taskGroups' => $taskGroups,
            'totals' => compact('day', 'week', 'month', 'ytd')
        ];
    }
}
