<?php

namespace App\Http\Controllers;

use App\UltiproImport;
use App\Imports\Ultipro;
use App\Mail\UltiproImportDetails;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;

use Storage;
use Maatwebsite\Excel\Facades\Excel;
use Carbon\Carbon;

class UltiproImportController extends Controller
{
    public function __construct()
    {
        $this->sftpDir = '/private/ultimatesoftware/';
        $this->localDir = 'ultipro/';
        $this->todaysFile = 'UP_'. date('Ymd') .'.csv';
        //$this->todaysFile = 'UP_'. date('Ymd', strtotime('-1 days')) .'.csv';
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $transfer = self::transfer($this->todaysFile);

        if ($transfer['success'] === true) {
            self::destroy($this->todaysFile);
            self::import($this->todaysFile);
        } elseif ($transfer['success'] === false) {
            dd($transfer);
        }
        echo 'import complete';
        Log::notice('Import complete.');
    }

    public function transfer()
    {
        $searchForFile = 'UP_' . date('Ymd');
        $source = $this->sftpDir . $this->todaysFile;
        $local =  $this->localDir . $this->todaysFile;

        if (Storage::disk('sftp')->exists($source) === true) {
            $importFile = Storage::disk('sftp')->get($source);
            Storage::disk('local')->put($local, $importFile);

            if (Storage::disk('local')->exists($local) === true) {
                return [
                    'file' => $this->todaysFile,
                    'success' => true,
                    'message' => 'File ('. $source .') successfully transfered localy to ('. $local .').'
                ];
            } else {
                return [
                    'file' => $this->todaysFile,
                    'success' => false,
                    'error' => 'Unable to transfer file ('. $source .').'
                ];
            }
        } else {
            return [
                'file' => $this->todaysFile,
                'success' => false,
                'error' => 'File ('. $source .') not found.'
            ];
        }
    }

    public function getFile(Request $request, $file)
    {
        $hash = $request->h;
        if (Hash::check($file, $hash)) {
            $path = storage_path('app/ultipro/'. $file);
            return response()->download($path, $file, ['Content-Type', 'text/csv']);
        }
    }

    public function import()
    {
        $import = (new Ultipro())->fromFile($this->todaysFile);
        Excel::import($import, $this->localDir . $this->todaysFile);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  str  $this->todaysFile
     * @return \Illuminate\Http\Response
     */
    public function destroy()
    {
        $event = UltiproImport::where('file', $this->todaysFile)->delete();
    }

}
