<?php

namespace App\Http\Controllers;

use App\UltiproImport;
use App\Imports\Ultipro;
use App\Mail\UltiproImportDetails;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;

use Storage;
use Maatwebsite\Excel\Facades\Excel;
use Carbon\Carbon;

class UltiproImportToUserController extends Controller
{
    public function __construct()
    {
        $this->sftpDir = '/private/ultimatesoftware/';
        $this->localDir = 'ultipro/';
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $fileName = 'UP_TEST_20190920105208.csv';

        $transfer = self::transfer($fileName);

        if ($transfer['success'] === true) {
            self::destroy($fileName);
            self::import($fileName);
        } elseif ($transfer['success'] === false) {
            dd($transfer);
        }
    }

    public function transfer($fileName)
    {
        //$sftp = Storage::disk('sftp')->allFiles('/private/ultimatesoftware');
        
        $source = $this->sftpDir . $fileName;
        $local =  $this->localDir . $fileName;

        if (Storage::disk('sftp')->exists($source) === true) {
            $importFile = Storage::disk('sftp')->get($source);
            Storage::disk('local')->put($local, $importFile);

            if (Storage::disk('local')->exists($local) === true) {
                return [
                    'file' => $fileName, 
                    'success' => true,
                    'message' => 'File ('. $source .') successfully transfered localy to ('. $local .').'
                ];
            } else {
                return [
                    'file' => $fileName, 
                    'success' => false,
                    'error' => 'Unable to transfer file ('. $source .').'
                ];
            }
        } else {
            return [
                'file' => $fileName, 
                'success' => false,
                'error' => 'File ('. $source .') not found.'
            ];
        }
    }

    public function import($fileName) 
    {
        $import = (new Ultipro())->fromFile($fileName);
        Excel::import($import, $this->localDir . $fileName);
    }

    public function postImport() //$fileName
    {

        $user = (object) [];
        $fileName = 'UP_TEST_20190920105208.csv';

        $import = UltiproImport::with('userMeta')->where('file', $fileName)->get()->groupBy('employee_number');

        $import->each(function($item) {
            $item = $item->first();
            if (empty($item->userMeta)) { // && $item->userMeta->employee_number === null
                $userMeta = self::searchByName($item->employee);
                $userMeta = $userMeta['userMeta'];
            } else {
                $userMeta = $item->userMeta;
            }

            if ($userMeta === false) {
                self::newUser($item);
            }

            if ($userMeta !== false) {
                self::setUserEmployeeInfo($userMeta, $item);
            }
        });

        self::terminate($fileName);
        self::mail($fileName);
        echo 'done';
    }

    public function importDetails($fileName)
    {
        $now = new Carbon;

        $import = UltiproImport::with('userMeta')->where('file', $fileName)->get();

        $stats['fileName'] = $fileName;
        $stats['initiated'] = $import->first()->import_initiated;
        $stats['total'] = $import->count();
        $stats['totalGrouped'] = $import->groupBy('employee_number')->count();

        $stats['newAccounts'] = \App\User::with('userMeta')
            ->where('created_at', '>=', $now->subMinute(2)->toDateTimeString())
            ->get();
        $stats['newAccountsTotal'] = $stats['newAccounts']->count();
        
        $stats['updatedAccounts'] = \App\UserMeta::with('legacy')
            ->where('last_import_change', '>=', $now->subMinute(2)->toDateTimeString())
            ->get();
        $stats['updatedAccountsTotal'] = $stats['updatedAccounts']->count();

        $stats['terminatedAccounts'] = \App\UserMeta::with('legacy')
            ->whereNotNull('term_date')
            ->where('last_import_change', '>=', $now->subMinute(2)->toDateTimeString())
            ->get();
        $stats['terminatedAccountsTotal'] = $stats['terminatedAccounts']->count();

        return $stats;
    }

    public function mail($fileName)
    {
        $details = self::importDetails($fileName);
        Mail::to('dhopkins@sailingstones.com')
            ->send(new UltiproImportDetails($details));
    }

    public function terminate($fileName)
    {
        $import = UltiproImport::with('userMeta')
            ->where('file', $fileName)
            ->whereNotNull('term_date')
            ->get();

        foreach($import as $item) {
            $legacy = \App\UserLegacy::findOrFail($item->userMeta->user_id);
            $legacy->active = 0;
            $legacy->save();

            $item->userMeta->term_date = Carbon::parse($item['term_date'])->toDateString();
            $item->userMeta->save();
        }
    }

    public function searchByName($name)
    {
        $message = '?';
        $found = false;

        $nameParts = explode(', ', $name);
        $lastName = explode(' ', $nameParts[0]);
        $firstName = explode(' ', $nameParts[1]);

        $search = \App\UserMeta::where('full_name', 'like', '%' . $lastName[0])
            ->where('full_name', 'like', '%' . $firstName[0] . '%')
            ->whereNull('employee_number')
            ->with('legacy')
            ->get();

        if ($search->count() === 0) { // create new user
            $found = false;
            $message = $name . ' not found. New user will be created.';
        } else {
            $message = 'Found a user ' . $name . ' that has not been updated yet.';
        }

        echo $message .'<br>';

        return ['userMeta' => $found, 'message' => $message];
    }

    public function newUser($item)
    {
        $password = $item['employee_number'] . $item['employee_number'];
        $username = self::uniqueUsername($item['employee']);

        $station = \App\Station::where('abbr', $item['station'])->get()->first();
        $stationIds = json_encode([$station->id]);

        $legacy = new \App\UserLegacy;
        $legacy->group_id = 4;
        $legacy->created_on = time();
        $legacy->ip_address = $_SERVER['REMOTE_ADDR']; //Request::ip();
        $legacy->username = $username;
        $legacy->password = self::hash_password($password);
        $legacy->email = self::uniqueEmail($item['employee']);
        $legacy->active = 1;

        $legacy->save();

        $user = new \App\User; // user_new
        $user->legacy_id = $legacy->id;
        $user->username = $username;
        $user->email = self::uniqueEmail($item['employee']);
        $user->password = Hash::make($password);

        $user->save();

        $userMeta = new \App\UserMeta;
        $userMeta->user_id = $legacy->id;
        $userMeta->full_name = $item['employee'];
        $userMeta->employee_number = $item['employee_number'];
        $userMeta->job = $item['job'];
        $userMeta->stations = $stationIds;
        $userMeta->station = $item['station'];
        $userMeta->rate = $item['rate'];
        $userMeta->hire_date = Carbon::parse($item['hire_date'])->toDateString();

        $userMeta->save();
    }

    public function setUserEmployeeInfo($userMeta, $item)
    {
        $legacy = \App\UserLegacy::findOrFail($userMeta->user_id);
        $legacy->active = 1;
        $legacy->save();

        $station = \App\Station::where('abbr', $item['station'])->get()->first();
        $stationIds = json_encode([$station->id]);

        //$userMeta = \App\UserMeta::with('legacy')->where('user_id', $user->id)->get()->first();
        $userMeta->employee_number = $item['employee_number'];
        $userMeta->job = $item['job'];
        $userMeta->stations = $stationIds;
        $userMeta->station = $item['station'];
        $userMeta->rate = $item['rate'];
        $userMeta->hire_date = Carbon::parse($item['hire_date'])->toDateString();

        $userMeta->save();
    }

    public function uniqueUsername($name)
    {
        $nameParts = explode(', ', $name);
        $lastName = explode(' ', $nameParts[0]);
        $firstName = explode(' ', $nameParts[1]);

        return strtolower($firstName[0] . $lastName[0]);
    }

    public function uniqueEmail($name)
    {
        $nameParts = explode(', ', $name);
        $lastName = explode(' ', $nameParts[0]);
        $firstName = explode(' ', $nameParts[1]);

        return strtolower($firstName[0] . '.' . $lastName[0]) . '@accufleet.com';
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  str  $fileName
     * @return \Illuminate\Http\Response
     */
    public function destroy($fileName)
    {
        $event = UltiproImport::where('file', $fileName)->delete();
    }


    /**
     * Legacy password hash
     * Misc functions
     *
     * Hash password : Hashes the password to be stored in the database.
     * Hash password db : This function takes a password and validates it
     * against an entry in the users table.
     * Salt : Generates a random salt value.
     *
     * @author Mathew
     */

    /**
     * Hashes the password to be stored in the database.
     *
     * @return void
     * @author Mathew
     **/
    public function hash_password($password, $salt=false)
    {
        if (empty($password)) {
            return FALSE;
        }

        $salt = $this->salt();
        return  $salt . substr(sha1($salt . $password), 0, -10);
    }

    /**
     * Generates a random salt value.
     *
     * @return void
     * @author Mathew
     **/
    public function salt()
    {
        return substr(md5(uniqid(rand(), true)), 0, 10);
    }

}
