<?php

namespace App\Listeners;

use App\ServiceLog;
use FFMpeg\FFMpeg;
use Illuminate\Http\File;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Spatie\MediaLibrary\Events\MediaHasBeenAdded;
use Spatie\MediaLibrary\Helpers\File as MediaLibraryFileHelper;
use Storage;

class ConvertVideo
{
    protected object $media;

    /**
     * Create the event listener.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Handle the event.
     *
     * @param MediaHasBeenAdded $event
     * @return void
     */
    public function handle(MediaHasBeenAdded $event)
    {
        $this->media = $event->media;

        if ($this->media->mime_type !== 'video/quicktime') {
            return;
        }

        $serviceLog = ServiceLog::find($this->media->model_id);

        info("Start video conversion");

        $convertedFile = $this->media->name . '.mp4';

            $ffmpeg = FFMpeg::create([
            'ffmpeg.binaries'  => '/usr/bin/ffmpeg',
            'ffprobe.binaries' => '/usr/bin/ffprobe',
        ]);
        $video = $ffmpeg->open($this->media->getPath());
        $video
            ->save(new \FFMpeg\Format\Video\X264(), 'videos/converted/'.$convertedFile);

        /*
         * This works
        Storage::disk('s3')->putFileAs($this->mediaLibraryPath(),
            new File(public_path().'/videos/converted/'.$convertedFile),
            $convertedFile
        );
        */

        ServiceLog::find($this->media->model_id)
            ->addMedia(public_path().'/videos/converted/'.$convertedFile)
            ->withCustomProperties(['grouping' => $this->media->getCustomProperty('grouping') ?? null])
            ->toMediaCollection($serviceLog->serviceType->uri);

        info("Stop video conversion");
    }

    public function mediaLibraryPath(): string
    {
        if (app()->isProduction()) {
            return 'media-library/' . $this->media->id;
        }
        return 'media-library-local/' . $this->media->id;
    }

    public function thumbName(): string
    {
        return $this->media->name . '.jpg';
    }
}
