<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use App\Filters\ServiceLog\ServiceLogFilters;
use App\Filters\Filters;

use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\MediaLibrary\Models\Media;
use Spatie\MediaLibrary\HasMedia\HasMedia;
use Spatie\MediaLibrary\HasMedia\HasMediaTrait;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ServiceLog extends Model implements HasMedia
{
    use SoftDeletes;
    use HasMediaTrait;

    protected $guarded = [
        //'user_id',
        'id',
        'modified_date',
        'entry_date',
    ];

    public static function exteriorWash()
    {
        //return self::serviceType(1);
    }

    public function scopeServiceType($query, $serviceType)
    {
        return $query->where('service_type_id', $serviceType);
    }

    public function serviceType()
    {
        return $this->belongsTo('App\ServiceType');
    }

    public function station()
    {
        return $this->belongsTo('App\Station');
    }

    public function airline()
    {
        return $this->belongsTo('App\Airline');
    }

    public function fleet()
    {
        return $this->belongsTo('App\Fleet', 'tail_number_id', 'id')->withTrashed();
    }

    public function task()
    {
        return $this->belongsTo('App\Task');
    }
    public function gate()
    {
        return $this->belongsTo('App\Gate');
    }

    public function user()
    {
        return $this->belongsTo('App\User');
    }

    /*
    public function userLegacy()
    {
        return $this->belongsTo('App\UserLegacy', '', 'user_id');
    }
    */

    public function openedBy()
    {
        return $this->belongsTo('App\UserMeta', 'opened_by', 'user_id');
    }

    public function closedBy()
    {
        return $this->belongsTo('App\UserMeta', 'closed_by', 'user_id');
    }

    public function userMeta()
    {
        return $this->belongsTo('App\UserMeta', 'user_id', 'user_id');
    }

    public function employee()
    {
        return $this->belongsTo('App\Employee', 'reference_id');
    }

    public function geo()
    {
        return $this->belongsTo('App\Geo', 'geo_id', 'id');
    }

    public function scopeOrder(Builder $builder, Request $request)
    {
        if (!isset($request['order']) || !isset($request['dir'])) {
            return $builder->orderBy('start', 'desc');
        } else {
            return $builder->orderBy($request->order, $request->dir);
        }
    }

    public function scopeFilter(Builder $builder, Request $request, array $filters = [])
    {
        return (new ServiceLogFilters(request()))->add($filters)->filter($builder);
    }

    public function registerMediaCollections()
    {
        $this
            ->addMediaCollection('original-video')
            ->useDisk('local');
    }

    public function registerMediaConversions(Media $media = null)
    {
        $this->addMediaConversion('thumb')
            ->fit('crop', 300, 300)
            ->sharpen(10);
    }

    public static function withJoins()
    {
        $joins = DB::table('service_logs')
            ->select(
                'service_logs.*, fleet.tail_number,
                tasks.title as task, tasks.cycle',
                DB::raw('TIME_TO_SEC(service_logs.stop) - TIME_TO_SEC(service_logs.start) AS duration')
            )
            ->join('fleet', 'service_logs.tail_number_id', '=', 'fleet.id')
            ->join('tasks', 'service_logs.task_id', '=', 'tasks.id')
            //->join('task_groups', 'service_logs.builder_id', '=', 'task_groups.id')
            ->join('airlines', 'service_logs.airline_id', '=', 'airlines.id')
            //->join('aircraft_types', 'service_logs.inspector_id', '=', 'aircraft_types.id')
            ->join('gates', 'service_logs.gate_id', '=', 'gates.id')
            //->join('barcodes', 'service_logs.inspection_status_id', '=', 'barcodes.id')
            ->join('stations', 'service_logs.station_id', '=', 'stations.id')
            ->join('user_meta', 'service_logs.user_id', '=', 'user_meta.id');

        return $joins;
    }
}
