<?php

namespace App\Transformers;

use App\Schedule;

class ComplianceTransformer extends \League\Fractal\TransformerAbstract
{
    protected $defaultIncludes = ['task', 'fleet', 'airline', 'aircraftType', 'gate'];

    public function transform(Schedule $schedule)
    {
        //dd($schedule);

        $scheduled = strtotime($schedule->date);
        $status = false;
        
        if (($schedule->task->id == 532 || $schedule->task->id == 533) && $scheduled >= strtotime('-3650 days')) {
            $status = 'ready';
        } elseif ($scheduled < strtotime('-1 day')) {
            $status = 'overdue';
        } elseif ($scheduled <= strtotime('+180 days')) {
            $status = 'ready';
        }
        $transforms = [
            'id' => $schedule->id,
            'service_type' => $schedule->serviceType->title,
            'service_type_id' => $schedule->service_type_id,
            'scheduled_date' => $schedule->date,
            'date' => date('m/d', strtotime($schedule->date)),
            'clean_id' => $schedule->clean_id,
            'clean_date' => $schedule->clean_date,
            'clean_todo' => $schedule->clean_todo,
            'mechanical_id' => $schedule->mechanical_id,
            'mechanical_date' => $schedule->mechanical_date,
            'mechanical_todo' => $schedule->mechanical_todo,
            'electrical_id' => $schedule->electrical_id,
            'electrical_date' => $schedule->electrical_date,
            'electrical_todo' => $schedule->electrical_todo,
            'status' => $status,
            'todo_id' => $schedule->todo['id'],
            'barcode' => $schedule->barcode['barcode'],
        ];
        
        return $transforms;
    }
    
    public function includeTask(Schedule $schedule)
    {
        return $this->item($schedule->task, new TaskTransformer);
    }
    
    public function includeFleet(Schedule $schedule)
    {
        return $this->item($schedule->fleet, new FleetTransformer);
    }

    public function includeAircraftType(Schedule $schedule)
    {
        return $this->item($schedule->fleet->aircraftType, new AircraftTypeTransformer);
    }

    public function includeGate(Schedule $schedule)
    {
        if ($schedule->fleet->gate !== null) {
            return $this->item($schedule->fleet->gate, new GateTransformer);
        }
    }

    public function includeAirline(Schedule $schedule)
    {
        return $this->item($schedule->fleet->aircraftType->airline, new AirlineTransformer);
    }
}
