<?php

namespace App\Transformers;

use App\ServiceLog;

class ServiceLogEloquentTransformer extends \League\Fractal\TransformerAbstract
{
    protected $defaultIncludes = ['task', 'fleet', 'airline', 'aircraftType', 'gate', 'station', 'geo', 'threshold', 'turnaround'];

    public function transform(ServiceLog $serviceLog)
    {
        $transforms = [
            'id' => $serviceLog->id,
            'created_at' => $serviceLog->created_at,
            'user_full_name' => $serviceLog->userMeta->full_name,
            'user_id' => $serviceLog->user_id,
            'opened_by' => $this->openedBy($serviceLog),
            'closed_by' => $this->closedBy($serviceLog),
            'start' => $serviceLog->start,
            'stop' => $serviceLog->stop,
            'duration' => $serviceLog->duration,
            'turnaround' => $this->includeTurnaround($serviceLog),
            'scheduled_date' => $serviceLog->scheduled_date,
            'status' => $serviceLog->status,
            'note' => $serviceLog->note,
            'utp' => $serviceLog->utp_reason,
            'media' => $this->media($serviceLog),
        ];
        
        return $transforms;
    }

    public function openedBy(ServiceLog $serviceLog)
    {
        if (isset($serviceLog->openedBy)) {
            return $serviceLog->openedBy->full_name;
        }
    }

    public function closedBy(ServiceLog $serviceLog)
    {
        if (isset($serviceLog->closedBy)) {
            return $serviceLog->closedBy->full_name;
        }
    }

    public function includeTask(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog->task, new TaskTransformer);
    }
    
    public function includeFleet(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog->fleet, new FleetTransformer);
    }

    public function includeAircraftType(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog->fleet->aircraftType, new AircraftTypeTransformer);
    }

    public function includeGate(ServiceLog $serviceLog)
    {
        if ($serviceLog->gate !== null) {
            return $this->item($serviceLog->gate, new GateTransformer);
        }
    }

    public function includeAirline(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog->fleet->aircraftType->airline, new AirlineTransformer);
    }

    public function includeStation(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog->station, new StationTransformer);
    }

    public function includeGeo(ServiceLog $serviceLog)
    {
        if ($serviceLog->geo !== null) {
            return $this->item($serviceLog->geo, new GeoTransformer);
        }
    }

    public function includeThreshold(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog, new ThresholdTransformer);
    }

    public function includeTurnaround(ServiceLog $serviceLog)
    {
        if (isset($serviceLog->status) && $serviceLog->status > 0) {
            return $this->item($serviceLog, new TurnaroundTransformer);
        }
    }

    public function media(ServiceLog $serviceLog)
    {
        $media = [];
        foreach ($serviceLog->media as $item) {
            $full[] = $item->getFullUrl();
            $thumbs[] = $item->getFullUrl('thumb');
        }
        if (isset($full)) {
            return $media = ['thumbs' => $thumbs, 'full' => $full];
        }
    }
}
