<?php

namespace App\Transformers;

use App\ServiceLog;

class TrainingLogTransformer extends \League\Fractal\TransformerAbstract
{
    protected $defaultIncludes = ['employee', 'task'];

    public function transform(ServiceLog $serviceLog)
    {
        $transforms = [
            'id' => $serviceLog->id,
            'created_at_detail' => $serviceLog->created_at,
            'created_at' => $serviceLog->created_at->toDateTimeString(),
            'created_human' => $serviceLog->created_at->diffForHumans(),
            'updated_at_detail' => $serviceLog->updated_at,
            'updated_at' => $serviceLog->updated_at->toDateTimeString(),
            'updated_at_human' => $serviceLog->updated_at->diffForHumans(),
            'user_full_name' => $serviceLog->userMeta->full_name,
            'user_id' => $serviceLog->user_id,
            'service_type' => $serviceLog->serviceType->title,
            'service_type_id' => $serviceLog->service_type_id,
            'station' => $serviceLog->station->title,
            'station_id' => $serviceLog->station_id,
            'start_local' => $serviceLog->start,
            'start_central' => $serviceLog->start_c,
            'stop_local' => $serviceLog->stop,
            'stop_central' => $serviceLog->stop_c,
        ];

        $needed = [
            'service_crew_size', //filter
            'scheduled_date',
            'note'
        ];

        $remaining = array_intersect_key($serviceLog->toArray(), array_flip($needed));

        $data = array_merge($transforms, $remaining);
        return $data;
    }

    public function includeEmployee(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog->employee, new EmployeeTransformer);
    }
    public function includeTask(ServiceLog $serviceLog)
    {
        return $this->item($serviceLog->task, new TaskTransformer);
    }
}
